/*

    GDS7 v0.1 interactive digital image transfer software for DS-7 camera
    Copyright (C) 1998 Matthew G. Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/
#include "gds7.h"
#include "ds7lib.h"
#include "ops.h"
#include "ui.h"
#include "callbacks.h"
#include <stdio.h>
#include <sys/param.h>

/***********************************************************************
 Callbacks
**********************************************************************/
/* Update the progress bar */
void progcb(float p){
  gtk_progress_bar_update(GTK_PROGRESS_BAR(progress),p);
  /* Force a screen update */
  gtk_sync();
};

/* Kill a dialog box */
void killdialog(GtkWidget *calling,gpointer data){
  GtkWidget *killthis=(GtkWidget *)data;
  gtk_widget_destroy(killthis);
};

/* Download a picture */
void getpic(struct picent *pic){
  if (!get_pic(pic->picnum,gtk_entry_get_text(GTK_ENTRY(pic->namebox)),progcb))
    return;
  printf("Picture %d saved to %s\n",pic->picnum,
	 gtk_entry_get_text(GTK_ENTRY(pic->namebox)));
  pic->status|=STAT_SAVED;
  color_selected(pic);
};

/* Save-as dialog box callback */
void dialog_save(GtkWidget *widget, GtkFileSelection *data){
  struct picent *pic=poppic;
  gtk_entry_set_text(GTK_ENTRY(pic->namebox),
		 gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
  getpic(pic);
};

/* Save-as dialog */
void saveas(GtkWidget *widget, gpointer *data){
  int i,getall;

  GtkWidget *dialog;

  dialog=gtk_file_selection_new("Select Save File");

  gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(dialog)->cancel_button),
		      "clicked",(GtkSignalFunc) killdialog, dialog);
  gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
		      "clicked",(GtkSignalFunc) dialog_save, dialog);
  gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(dialog)->ok_button),
		      "clicked",(GtkSignalFunc) killdialog, dialog);

  gtk_widget_show(dialog);
};
   
/* another callback */
void delete_event (GtkWidget *widget, gpointer *data)
{    
    gtk_main_quit ();
}

/* Thumbnail pop-up menu */
void pop_thumb(GtkWidget *widget, gpointer data){
  if (!(poppic->status&STAT_THUMB))
    thumbtrans(poppic->picnum);
};

/* Get picture: callback from thumbnail menu */
void pop_get(GtkWidget *widget, gpointer data){
  getpic(poppic);
};

/* Config Baudrate */
void baudset(GtkWidget *widget, gpointer data){
  char *buffer;
  gtk_label_get(GTK_LABEL(data),&buffer);
  menubaud=atoi(buffer);
};

/* Apply new config info */
void config_update(GtkWidget *widget, gpointer data){
  free(preview_cmd);
  preview_cmd= strdup(gtk_entry_get_text(GTK_ENTRY(prev_sel)));
  free(camdev);
  camdev= strdup(gtk_entry_get_text(GTK_ENTRY(dev_sel)));
  thumbscale=gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(size_sel));
  thumbgam=gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(gam_sel));
  cols=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(col_sel));
  baudrate=menubaud;
  save_cfg();
  /* init will wipe out all thumbnail info... */
  cam_init(NULL,NULL);
};

/* Not that the save-filename has changed,revert to unsaved */
void fname_changed(GtkWidget *widget, gpointer data){
  poppic->status&=~STAT_SAVED;
  color_selected(poppic);
};

/*
  Change the color of the text entry box depending on the image status
 */
void color_selected(struct picent *pic){
  if (pic->status&STAT_SAVED){
    gtk_widget_set_name(pic->namebox,"saved_text");
    gtk_widget_set_rc_style(pic->namebox);
  }
  else
    if (is_selected(pic->picnum)) 
        {
          gtk_widget_set_name(pic->namebox,"selected_text");
          gtk_widget_set_rc_style(pic->namebox);
        }
    else {
          gtk_widget_set_name(pic->namebox,"filetext");
          gtk_widget_set_rc_style(pic->namebox);
    };
};

/*
  Handle thumbnail button toggle
*/
void thumb_toggle(GtkWidget *widget,gpointer data){
  struct picent *pic=poppic;
      color_selected(pic);
};

/* 
   Handle a click on a thumbnail image
*/
int thumbclick(GtkWidget *widget,GdkEventButton *event,gpointer data){
  struct picent *pic=data;
  switch(event->button){
  case(1):
    if (event->type==GDK_2BUTTON_PRESS){
      if (!(poppic->status&STAT_THUMB))
	thumbtrans(pic->picnum);
      return TRUE;
    };
    break;
  case (3):
    poppic=pic;
    gtk_menu_popup(GTK_MENU(popmenu),NULL,NULL,NULL,pic
		   ,event->button,event->time);
    return TRUE;
  };
};


/* 
   Record entry of cursor into a thumbnail
*/
int thumbenter(GtkWidget *widget,GdkEvent *event,gpointer data){
  struct picent *pic=data;
  poppic=pic;
  /* Don't hog the event */
  return FALSE;
};

/*
  Handle a cancel request
*/
void cancel_toggle(GtkWidget *widget,gpointer data){
  req_cancel();
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widget),0);
};

/*
  Configuration File routines, cheap and dirty
*/

int load_cfg(){
  char inbuf[1000];
  FILE *fp;
  char inpath[MAXPATHLEN];

  strncpy(inpath,getenv("HOME"),MAXPATHLEN);
  strcat(inpath,"/.gds7rc");

#ifdef DEBUG
  printf("RCfile is %s\n",inpath);
#endif

  if ((fp=fopen(inpath,"r"))==NULL){
    /* Defaults */
    
    cols=5;
    thumbscale=1.5;
    thumbgam=2;
    baudrate=38400;
    camdev=strdup("/dev/ttyS1");
    preview_cmd=strdup("ee");
    printf("Config file not found, writing new\n");
    save_cfg();
    return(1);
  };

  /* Parse the file */
    fgets(inbuf,1000,fp);
    if (inbuf[strlen(inbuf)-1]=='\n') inbuf[strlen(inbuf)-1]='\0';
  while(!feof(fp)){
    if (strstr(inbuf,"camdev")!=NULL){
      camdev=strdup(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found camdev=%s\n",camdev);
#endif
    }
    else if (strstr(inbuf,"previewcmd")!=NULL){
      preview_cmd=strdup(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found prvstr=%s\n",preview_cmd);
#endif
    }
    else if (strstr(inbuf,"baudrate")!=NULL){
      baudrate=atoi(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found baudrate=%d\n",baudrate);
#endif
    }
    else if (strstr(inbuf,"thumbscale")!=NULL){
      thumbscale=atof(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found thumbscale=%.1f\n",thumbscale);
#endif
    }
    else if (strstr(inbuf,"thumbgamma")!=NULL){
      thumbgam=atof(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found thumbgam=%.1f\n",thumbgam);
#endif
    }
    else if (strstr(inbuf,"columns")!=NULL){
      cols=atoi(strchr(inbuf,':')+2);
#ifdef DEBUG
      printf("Found cols=%d\n",cols);
#endif
    }
    fgets(inbuf,1000,fp);
    if (inbuf[strlen(inbuf)-1]=='\n') inbuf[strlen(inbuf)-1]='\0';
  };
  fclose(fp);

  return(0);
};

int save_cfg(){
  FILE *fp;
  char outpath[MAXPATHLEN];

  strncpy(outpath,getenv("HOME"),MAXPATHLEN);
  strcat(outpath,"/.gds7rc");

    if ((fp=fopen(outpath,"w"))!=NULL){
      fprintf(fp,"camdev: %s\n",camdev);
      fprintf(fp,"previewcmd: %s\n",preview_cmd);
      fprintf(fp,"baudrate: %d\n",baudrate);
      fprintf(fp,"thumbscale: %.1f\n",thumbscale);
      fprintf(fp,"thumbgamma: %.1f\n",thumbgam);
      fprintf(fp,"columns: %d\n",cols);
      fclose(fp);
      return(0);
    };
  printf("Couldn't write config file !\n");
  return(1);
};

/* 
   Display picture info
 */
void thumb_get_info(GtkWidget *widget, gpointer data){
  struct picent *thepic=poppic;
  char* info=get_info(poppic->picnum);

  popup_dialog(info);
  free(info);
};

/*
  Help Box, simple text for now
*/
void show_help(GtkWidget *widget, gpointer data){
  help_dialog();
};
